#!/usr/bin/python
#
# ddprog.py

import os
import sys
import subprocess
import time
import signal
import select

def print_estimate(pos, size, time, start):
    if size == 0:
        print "\r%dMB" % (pos / 1E6),
    else:
        pct = (pos * 100) / size
        bar = "#" * (pct  / 5)
        bar += "-" * (20 - len(bar))
        lapse = time - start
        cps = pos / lapse
        if cps == 0:
            est = -1
        else:
            est = (size / cps)
        speed = "%02d:%02d / %02d:%02d" % (
            lapse / 60, lapse % 60,
            est / 60, est % 60)
            
        print "\r[%s] %dMB/%dMB [%d%%] %s" % (bar,
            pos / 1E6,size / 1E6,
            pct, speed),
        

def dd_info(f, filesize, start):
    pos = -1
    rin = f.readline()
    rout = f.readline()
    rt = f.readline()
    
    if rt != "":
        pos = int(rt.split()[0])
        print_estimate(pos, filesize, time.time(), start)
    
    return pos

def do_copy(infile, outfile, sig, bs):
    
    if not os.path.exists(infile):
        print "File is not found!"
        return
    
    filesize = os.path.getsize(infile)
   
    args = [
        "/bin/dd",
        "if=%s" % infile,
        "of=%s" % outfile,
        "bs=%s" % bs]

    start = time.time()
    pos = 0
    
    p = subprocess.Popen(args, stderr=subprocess.PIPE)
 
    while p.poll() is None:
        os.kill(p.pid, sig)

        rlist = select.select([p.stderr], [], [], 0)[0]
        if len(rlist) > 0:
            pos = dd_info(p.stderr, filesize, start)
            
        time.sleep(0.25)
    
    while pos >= 0:
        pos = dd_info(p.stderr, filesize, start)

    print ""

#
# main
#
sig = signal.SIGINFO
bs = "10m"

# Linux(untested)
if os.uname()[0] != "Darwin":
    sig = signal.SIGUSR1
    bs = "10M"


if len(sys.argv) < 3:
    print "Usage %s infile outfile" % sys.argv[0]
    quit()
    
do_copy(sys.argv[1], sys.argv[2], sig, bs)
    
    
        

	
